@@ -21,8 +21,10 @@
 
 #include "call.h"
 #include "call-private.h"
+#include <gst/gst.h>
 #include <libringtoned/ringtoned.h>
 #include <libringtoned/utils.h>
+#include <pulse/pulseaudio.h>
 #include <telepathy-glib/util.h>
 #include "ringtoned-enums.h"
 
@@ -44,7 +46,6 @@ enum
     PROP_ACCOUNT,
     PROP_CONTACTS,
     PROP_RINGTONE_PATH,
-    PROP_CA_CONTEXT,
 };
 
 struct _RingtonedCallPrivate
@@ -59,8 +60,7 @@ struct _RingtonedCallPrivate
     GList *contacts;
 
     gchar *ringtone_path;
-    gboolean playing;
-    ca_context *ca_ctx;
+    GstElement *pipeline;
 };
 
 G_DEFINE_TYPE (RingtonedCall, ringtoned_call, G_TYPE_OBJECT)
@@ -69,75 +69,69 @@ RingtonedCall *
 ringtoned_call_new (TpChannel   *channel,
                     const gchar *vcard_field,
                     const gchar *identifier,
-                    McAccount   *account,
-                    ca_context  *ca_ctx)
+                    McAccount   *account)
 {
     return g_object_new (RINGTONED_TYPE_CALL,
             "channel", channel,
             "vcard-field", vcard_field,
             "identifier", identifier,
             "account", account,
-            "ca-context", ca_ctx,
             NULL);
 }
 
-static void play_real (RingtonedCall *self);
-
 static gboolean
-play_real_idle_cb (gpointer user_data)
+bus_watch_cb (GstBus     *bus,
+              GstMessage *msg,
+              gpointer    user_data)
 {
     RingtonedCall *self = user_data;
 
-    play_real (self);
-
-    return FALSE;
-}
-
-static void
-playback_finished_cb (ca_context *c,
-                      guint32     id,
-                      gint        error_code,
-                      gpointer    user_data)
-{
-    RingtonedCall *self = user_data;
-
-    if (error_code == CA_SUCCESS)
+    switch (GST_MESSAGE_TYPE (msg))
     {
-        DEBUG ("Repeating %s", self->priv->ringtone_path);
-        g_idle_add (play_real_idle_cb, self);
+        case GST_MESSAGE_EOS:
+            DEBUG ("Repeating");
+            gst_element_seek_simple (self->priv->pipeline, GST_FORMAT_TIME,
+                    GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT, 0);
+            break;
+
+        case GST_MESSAGE_ERROR:
+        {
+            GError *err = NULL;
+            gchar *dbg_info = NULL;
+
+            gst_message_parse_error (msg, &err, &dbg_info);
+
+            g_printerr ("Error from element %s: %s\n",
+                    GST_OBJECT_NAME (msg->src), err->message);
+            g_printerr ("Debugging info:\n%s\n",
+                    dbg_info ? dbg_info : "none");
+
+            gst_object_unref (self->priv->pipeline);
+            self->priv->pipeline = NULL;
+
+            g_error_free (err);
+            g_free (dbg_info);
+
+            break;
+        }
+
+        default:
+            break;
     }
-}
 
-static void
-play_real (RingtonedCall *self)
-{
-    ca_proplist *props;
-    const gchar *playable;
-    gint r;
-
-    g_return_if_fail (self->priv->ca_ctx);
-
-    ca_proplist_create (&props);
-    /* FIXME: Avoid recalculating this when repeating */
-    playable = ringtoned_decoder_get_playable_path (self->priv->ringtone_path);
-    ca_proplist_sets (props, CA_PROP_MEDIA_FILENAME, playable);
-    ca_proplist_sets (props, "ringtoned.role", "call");
-
-    DEBUG ("Actually playing %s", playable);
-
-    self->priv->playing = TRUE;
-
-    r = ca_context_play_full (self->priv->ca_ctx, ID_RINGTONE, props,
-            playback_finished_cb, self);
-    if (r != CA_SUCCESS)
-        DEBUG ("Playing error %d: %s", r, ca_strerror (r));
-
-    ca_proplist_destroy (props);
+    return TRUE;
 }
 
 void
 ringtoned_call_play_ringtone (RingtonedCall *self)
 {
+    pa_proplist *proplist;
+    gchar *playable;
+    GstElement *src;
+    GstElement *sink;
+    GstBus *bus;
+    GError *error = NULL;
+
     if (self->priv->status != RINGTONED_CALL_STATUS_RINGING)
     {
         DEBUG ("ringtoned_call_play_ringtone() called, but not in "
@@ -145,7 +139,7 @@ ringtoned_call_play_ringtone (RingtonedCall *self)
         return;
     }
 
-    if (self->priv->playing)
+    if (self->priv->pipeline)
     {
         DEBUG ("Already playing");
         return;
@@ -157,12 +151,6 @@ ringtoned_call_play_ringtone (RingtonedCall *self)
         return;
     }
 
-    if (!self->priv->ca_ctx)
-    {
-        DEBUG ("No canberra context, not playing");
-        return;
-    }
-
     if (ringtoned_profile_get_current_volume () == 0)
     {
         DEBUG ("Volume muted, not playing anything");
@@ -171,18 +159,52 @@ ringtoned_call_play_ringtone (RingtonedCall *self)
 
     DEBUG ("Playing %s", self->priv->ringtone_path);
 
-    play_real (self);
+    self->priv->pipeline = gst_parse_launch (
+            "filesrc name=src ! wavparse ! pulsesink name=sink", &error);
+    if (!self->priv->pipeline)
+    {
+        g_critical ("Cannot create the ringtone pipeline: %s",
+                error ? error->message : "unknown error");
+        g_clear_error (&error);
+        return;
+    }
+
+    /* Magical bits to make the ringtone play at the right volume */
+    proplist = pa_proplist_new();
+    pa_proplist_sets (proplist, "event.id", "phone-incoming-call");
+    pa_proplist_sets (proplist, "module-stream-restore.id",
+            "x-maemo-hildon-notify");
+
+    sink = gst_bin_get_by_name (GST_BIN (self->priv->pipeline), "sink");
+    g_assert (sink);
+    g_object_set (sink, "proplist", proplist, NULL);
+
+    src = gst_bin_get_by_name (GST_BIN (self->priv->pipeline), "src");
+    g_assert (src);
+    playable = ringtoned_decoder_get_playable_path (self->priv->ringtone_path);
+    g_object_set (src, "location", playable, NULL);
+
+    bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->pipeline));
+    gst_bus_add_watch (bus, bus_watch_cb, self);
+
+    DEBUG ("Actually playing %s", playable);
+    gst_element_set_state (self->priv->pipeline, GST_STATE_PLAYING);
+
+    gst_object_unref (bus);
+    g_free (playable);
+    gst_object_unref (sink);
+    gst_object_unref (src);
 }
 
 static void
 stop_ringtone (RingtonedCall *self)
 {
-    if (self->priv->playing)
+    if (self->priv->pipeline)
     {
         DEBUG ("Stop playing %s", self->priv->ringtone_path);
-
-        ca_context_cancel (self->priv->ca_ctx, ID_RINGTONE);
-        self->priv->playing = FALSE;
+        gst_element_set_state (self->priv->pipeline, GST_STATE_NULL);
+        gst_object_unref (self->priv->pipeline);
+        self->priv->pipeline = NULL;
     }
 }
 
@@ -350,9 +372,6 @@ ringtoned_call_get_property (GObject    *object,
         case PROP_RINGTONE_PATH:
             g_value_set_string (value, self->priv->ringtone_path);
             break;
-        case PROP_CA_CONTEXT:
-            g_value_set_pointer (value, self->priv->ca_ctx);
-            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
             break;
@@ -409,10 +428,6 @@ ringtoned_call_set_property (GObject      *object,
             g_free (self->priv->ringtone_path);
             self->priv->ringtone_path = g_value_dup_string (value);
             break;
-        case PROP_CA_CONTEXT:
-            g_return_if_fail (self->priv->ca_ctx == NULL);
-            self->priv->ca_ctx = g_value_get_pointer (value);
-            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
             break;
@@ -578,16 +593,6 @@ ringtoned_call_class_init (RingtonedCallClass *class)
                 "",
                 G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-    g_object_class_install_property (
-            object_class,
-            PROP_CA_CONTEXT,
-            g_param_spec_pointer (
-                "ca-context",
-                "canberra context",
-                "The libcanberra context to use for the ringtone",
-                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                G_PARAM_STATIC_STRINGS));
-
     g_type_class_add_private (object_class, sizeof (RingtonedCallPrivate));
 }
 
